package view;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.RenderingHints;
import java.awt.geom.Arc2D;
import java.awt.geom.GeneralPath;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.net.URL;
import java.net.URLClassLoader;

import javax.imageio.ImageIO;
import javax.swing.BorderFactory;
import javax.swing.JFrame;
import javax.swing.JPanel;

import controler.MixedDisplayControler;
import model.HeadingChangedEvent;
import model.SpeedChangedEvent;

public class MixedDisplayUIView extends MixedDisplayView{

	protected static URLClassLoader urlLoader = (URLClassLoader)ClassLoader.getSystemClassLoader();
	protected static URL load(String file) {return urlLoader.findResource(file);}
	
	private JFrame frame=null;
	private MixedDisplayPane contentPane=null;
	private int heading,speed;
	
	public MixedDisplayUIView(MixedDisplayControler controler) {
		this(controler, 0, 0);
	}
	public MixedDisplayUIView(MixedDisplayControler controler, int heading, int speed){
		super(controler);
		this.heading=heading;
		this.speed=speed;
		buildFrame();
	}
	
	private void buildFrame(){
		frame=new JFrame();
		frame.setResizable(false);
		contentPane=new MixedDisplayPane("images/background.png");
		//contentPane.setBorder(BorderFactory.createTitledBorder(BorderFactory.createLineBorder(Color.RED),"Mixed Display"));

		frame.setContentPane(contentPane);
		frame.setBackground(Color.WHITE);
		frame.setTitle("Mixed Display");
		frame.pack();
	}
	
	public void close() {frame.dispose();}
	public void display() {frame.setVisible(true);}

	public void headingChanged(HeadingChangedEvent event) {
		heading=new Integer(event.getNewHeading());
		//en cas de changement de la valeur du cap, on provoque un re-dessin de l'image
		contentPane.repaint();
	}
	public void speedChanged(SpeedChangedEvent event) {
		speed=new Integer(event.getNewSpeed());
		//en cas de changement de la valeur de vitesse, on provoque un re-dessin de l'image
		contentPane.repaint();
	}
	
	class MixedDisplayPane extends JPanel{
		private static final long serialVersionUID = 1L;
		BufferedImage background=null;
		BufferedImage cursor=null;
		private double zoom=12.08;
		
		public MixedDisplayPane(String imageFile){
				//1
				//2
			//3
		}
		
		protected void paintComponent(Graphics g){
			Graphics2D g2d = (Graphics2D)g;
			g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
			//Insets inset = this.getBorder().getBorderInsets(this);
			
			//dessin du fond en blanc (effacement de la zone de dessin)
			g2d.setColor(Color.WHITE);
			g2d.fillRect(0,0,this.getWidth(),this.getHeight());
			
			//dessin de l'image de fond
			//4
			
			//position du curseur Vitesse-Cap
			//6
			//6
			
			//calcul de l'arc vers la barre de vitesse
			//9
			//10
			
			//triangle de fond pour l'aiguille de cap
			GeneralPath triangle1 = new GeneralPath(GeneralPath.WIND_EVEN_ODD, 3);
			//13
			//13
			//13
			//13
			//13
			
			//demi-triangle noir pour l'aiguille de cap
			GeneralPath triangle2 = new GeneralPath(GeneralPath.WIND_EVEN_ODD, 3);
			//16
			//16
			//16
			//16
			//16
			
			//dessin de l'aiguille de cap
			//14
			//14
			//15
			//15
			//17
			//17
			
			g2d.fillOval((int)(this.getWidth()/2-7),(int)(this.getHeight()/2-7),14,14);
			
			//dessin de l'arc vers la barre de vitesse
			//11
			//11
			
			//dessin du curseur Vitesse/Cap
			//7
			//7
			
			//8
			//8
			
			//dessin du curseur vitesse
			//11
			
		}
	}
}
