package view;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.geom.GeneralPath;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.BevelBorder;

import controler.MixedInputControler;
import model.HeadingChangedEvent;
import model.SpeedChangedEvent;

public class MixedInputUIView extends MixedInputView{

	private JFrame frame=null;
	private JPanel controlPanel=null;
	private JButton plus=null;
	private JButton minus=null;
	private ModalJPanel contentTop=null;
	private int heading,speed;
	
	
	
	public MixedInputUIView(MixedInputControler controler) {
		this(controler, 0, 0);
	}
	public MixedInputUIView(MixedInputControler controler, int heading, int speed){
		super(controler);
		this.heading=heading;
		this.speed=speed;
		buildFrame();
	}
	
	private void buildFrame(){
		frame=new JFrame();
		frame.setResizable(false);
		
		//boite des boutons +/-
		plus=new JButton("+");
		plus.addActionListener(new PlusActionListener());
		minus=new JButton("-");
		minus.addActionListener(new MinusActionListener());
		controlPanel=new JPanel();
		controlPanel.add(minus);
		controlPanel.add(plus);
		
		//boite modale pour l'affichage des labels
		contentTop=new ModalJPanel("<html><H1>"+heading+"&deg</H1></html>","<html><H1>"+speed+"nd</H1></html>");
		contentTop.addMouseListener(contentTop);
		
		//panel de fond
		JPanel contentPane=new JPanel();
		contentPane.setLayout(new BoxLayout(contentPane,BoxLayout.Y_AXIS));
		contentPane.add(contentTop);
		contentPane.add(controlPanel);
		
		//fenetre
		frame.setContentPane(contentPane);
		frame.setTitle("Boat Speed/heading setup");
		frame.pack();
	}
	
	
	public void close() {frame.dispose();}
	public void display() {frame.setVisible(true);}

	public void headingChanged(HeadingChangedEvent event) {
		heading=new Integer(event.getNewHeading());
		contentTop.setLeftLabelText("<html><H1>"+heading+"&deg</H1></html>");
	}
	public void speedChanged(SpeedChangedEvent event) {
		speed=new Integer(event.getNewSpeed());
		contentTop.setRightLabelText("<html><H1>"+speed+"nd</H1></html>");
	}
	
	class MinusActionListener implements ActionListener{
		public void actionPerformed(ActionEvent arg0){
			if(contentTop.isLeftSelected()){
				getControler().notifyHeadingSpeedChanged(heading-10,speed);
			}else{
				getControler().notifyHeadingSpeedChanged(heading,speed-1);
			}
		}
	}
	
	class PlusActionListener implements ActionListener{
		public void actionPerformed(ActionEvent arg0) {
			if(contentTop.isLeftSelected()){
				getControler().notifyHeadingSpeedChanged(heading+10,speed);
			}else{
				getControler().notifyHeadingSpeedChanged(heading,speed+1);
			}
		}
	}
	
	class ModalJPanel extends JPanel implements MouseListener{

		private static final long serialVersionUID = 1L;
		private JPanel arrow;
		private ModalJLabel l1,l2;
		
		public ModalJPanel(String s1, String s2){
			super();
			l1=new ModalJLabel(s1);
			l2=new ModalJLabel(s2);
			l2.setSelected(false);
			l1.setSelected(true);
			arrow=new JPanel(){
				private static final long serialVersionUID = 1L;
				public void paintComponent(Graphics g){
					Graphics2D g2d = (Graphics2D)g;
					g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
					g2d.setColor(Color.GREEN);
					
					g2d.fillRect(getWidth()/2-5,getHeight()/2-5,10,getHeight()/2-10);
					
					int inc=1;
					if(l1.isSelected()){inc=0;}
					g2d.fillRect(inc*getWidth()/2,getHeight()/2-5,getWidth()/2,10);
					
					int xPoints[]={this.getWidth()/2,this.getWidth()/2+15,this.getWidth()/2-15};
					int yPoints[]={this.getHeight(),this.getHeight()-20,this.getHeight()-20};
					GeneralPath triangle = new GeneralPath(GeneralPath.WIND_EVEN_ODD, xPoints.length);
					triangle.moveTo (xPoints[0], yPoints[0]);
					for (int index = 1; index < xPoints.length; index++) {triangle.lineTo(xPoints[index], yPoints[index]);};
					triangle.closePath();
					g2d.fill(triangle);
				}
			};
			arrow.setOpaque(false);
			arrow.setPreferredSize(new Dimension(50,100));
			
			setLayout(new BoxLayout(this,BoxLayout.LINE_AXIS));
			add(l1);
			add(arrow);
			add(l2);
		}
		public void switchButton(){
			l1.setSelected(!l1.isSelected());
			l2.setSelected(!l2.isSelected());
			arrow.repaint();
		}
		
		public boolean isLeftSelected(){
			return l1.isSelected();
		}
		public boolean isRightSelected(){
			return !isLeftSelected();
		}
		
		public void setLeftLabelText(String text){l1.setText(text);}
		public void setRightLabelText(String text){l2.setText(text);}
		
		public void mouseClicked(MouseEvent arg0) {switchButton();}
		public void mouseEntered(MouseEvent arg0) {}
		public void mouseExited(MouseEvent arg0) {}
		public void mousePressed(MouseEvent arg0) {}
		public void mouseReleased(MouseEvent arg0) {}
	}
	
	class ModalJLabel extends JPanel{
		private static final long serialVersionUID = 1L;
		Boolean selected=false;
		JLabel text;
		
		public ModalJLabel(String str){
			super();
			text=new JLabel(str);
			text.setHorizontalAlignment(JLabel.CENTER);
			text.setVerticalAlignment(JLabel.CENTER);
			text.setPreferredSize(new Dimension(100,100));
			add(text);
			setSelected(true);
		}
		public void setText(String str){
			text.setText(str);
		}

		public void setSelected(boolean selected){
			if(selected){	setBorder(BorderFactory.createLineBorder(Color.GREEN,2));		}
			else{			setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED));	}
			this.selected=selected;
		}
		public boolean isSelected(){return selected;}
	}
}
