package view;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.RenderingHints;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.geom.Arc2D;
import java.awt.geom.GeneralPath;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.net.URL;
import java.net.URLClassLoader;

import javax.imageio.ImageIO;
import javax.swing.BorderFactory;
import javax.swing.JFrame;
import javax.swing.JPanel;

import controler.MixedControler;
import model.HeadingChangedEvent;
import model.SpeedChangedEvent;

public class MixedUIView extends MixedView{

	protected static URLClassLoader urlLoader = (URLClassLoader)ClassLoader.getSystemClassLoader();
	protected static URL load(String file) {return urlLoader.findResource(file);}
	
	private JFrame frame=null;
	private MixedDisplayPane contentPane=null;
	private double heading,speed;
	
	public MixedUIView(MixedControler controler) {
		this(controler, 0, 0);
	}
	public MixedUIView(MixedControler controler, int heading, int speed){
		super(controler);
		this.heading=heading;
		this.speed=speed;
		buildFrame();
	}
	
	private void buildFrame(){
		frame=new JFrame();
		frame.setResizable(false);
		contentPane=new MixedDisplayPane("images/background.png");
		contentPane.setBorder(BorderFactory.createTitledBorder(BorderFactory.createLineBorder(Color.RED),"Mixed Display"));
		contentPane.addMouseListener(contentPane);
		contentPane.addMouseMotionListener(contentPane);
		
		
		frame.setContentPane(contentPane);
		frame.setBackground(Color.WHITE);
		frame.setTitle("Mixed Display");
		frame.pack();
	}
	
	public void close() {frame.dispose();}
	public void display() {frame.setVisible(true);}

	public void headingChanged(HeadingChangedEvent event) {
		heading=new Integer(event.getNewHeading());
		contentPane.repaint();
	}
	public void speedChanged(SpeedChangedEvent event) {
		speed=new Integer(event.getNewSpeed());
		contentPane.repaint();
	}
	
	
	
	class MixedDisplayPane extends JPanel implements MouseListener,MouseMotionListener{
		private static final long serialVersionUID = 1L;
		BufferedImage background=null;
		BufferedImage cursor=null;
		private double zoom=12.08;
		//contient la position du curseur VitesseCap
		private double x,y;
		//le curseur VC est il selectionné ?
		private boolean cursorSelected=false;
		
		public MixedDisplayPane(String imageFile){
			try {
				background=ImageIO.read(load(imageFile));
				cursor=ImageIO.read(load("images/cursor.png"));
			} catch (IOException e){
				e.printStackTrace();
			}
			this.setPreferredSize(new Dimension(background.getWidth(),background.getHeight()));
		}

		protected void paintComponent(Graphics g){
			Graphics2D g2d = (Graphics2D)g;
			g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
			Insets inset = this.getBorder().getBorderInsets(this);
			
			//dessin du fond en blanc (effacement de la zone de dessin)
			g2d.setColor(Color.WHITE);
			g2d.fillRect(0,0,this.getWidth(),this.getHeight());
			
			//dessin de l'image de fond
			g2d.drawImage(background,
					inset.left,inset.top,this.getWidth()-inset.right,this.getHeight()-inset.bottom,
					inset.left,inset.top,this.getWidth()-inset.right,this.getHeight()-inset.bottom,
					null);
			
			//position du curseur
			x=speed*zoom*Math.cos((heading-90)*(Math.PI/180))+this.getWidth()/2;
			y=speed*zoom*Math.sin((heading-90)*(Math.PI/180))+this.getHeight()/2;
			
			//calcul de l'arc vers la barre de vitesse
			Arc2D.Double arc=new Arc2D.Double();
			double alpha=(((heading+270-19)%360)-360)*-1;
			arc.setArcByCenter(
					this.getWidth()/2,this.getHeight()/2, 
					speed*zoom,
					-19,alpha,
					Arc2D.OPEN);
			
			//contour de l'aiguille de cap
			int x1Points[] = {(int)x, (int)(7*Math.cos((heading-180)*(Math.PI/180))+this.getWidth()/2), (int)(7*Math.cos((heading)*(Math.PI/180))+this.getWidth()/2)};
			int y1Points[] = {(int)y, (int)(7*Math.sin((heading-180)*(Math.PI/180))+this.getHeight()/2), (int)(7*Math.sin((heading)*(Math.PI/180))+this.getHeight()/2)};
			GeneralPath triangle1 = new GeneralPath(GeneralPath.WIND_EVEN_ODD, x1Points.length);
			triangle1.moveTo (x1Points[0], y1Points[0]);
			for (int index = 1; index < x1Points.length; index++) {triangle1.lineTo(x1Points[index], y1Points[index]);};
			triangle1.closePath();
			
			//partie noire de l'aiguille de cap
			x1Points[2]=this.getWidth()/2;
			y1Points[2]=this.getHeight()/2;
			GeneralPath triangle2 = new GeneralPath(GeneralPath.WIND_EVEN_ODD, x1Points.length);
			triangle2.moveTo (x1Points[0], y1Points[0]);
			for (int index = 1; index < x1Points.length; index++) {triangle2.lineTo(x1Points[index], y1Points[index]);};
			triangle2.closePath();
			
			//dessin de l'aiguille de cap
			g2d.fill(triangle1);
			g2d.setColor(Color.BLACK);
			g2d.fill(triangle2);
			g2d.draw(triangle1);
			g2d.fillOval((int)(this.getWidth()/2-7),(int)(this.getHeight()/2-7),14,14);
			
			//dessin de l'arc vers la barre de vitesse
			g2d.setColor(Color.RED);
			g2d.draw(arc);
			
			//dessin du curseur Vitesse/Cap
			g2d.fillOval((int)x-5, (int)y-5, 10, 10);
			g2d.setColor(Color.BLACK);
			g2d.drawOval((int)x-5, (int)y-5, 10, 10);
			
			//dessin du curseur vitesse
			g2d.drawImage(cursor,null,
					(int)(speed*zoom*Math.cos(19*(Math.PI/180))+this.getWidth()/2-1),
					(int)(speed*zoom*Math.sin(19*(Math.PI/180))+this.getHeight()/2-9));
			
		}
		
		//reaction aux évènements souris
		public void mouseClicked(MouseEvent arg0){}
		public void mouseEntered(MouseEvent arg0){}
		public void mouseExited(MouseEvent arg0){}
		public void mousePressed(MouseEvent arg0){
			if(Math.sqrt(Math.pow(x-arg0.getX(),2)+(Math.pow(y-arg0.getY(), 2)))<5){cursorSelected=true;}
		}
		public void mouseReleased(MouseEvent arg0){
			if(cursorSelected){cursorSelected=false;}
		}
		public void mouseDragged(MouseEvent arg0){
			if(cursorSelected){
				speed=Math.sqrt(Math.pow((getWidth()/2)-arg0.getX(), 2)+(Math.pow((getHeight()/2)-arg0.getY(),2)))/zoom;
				if((getWidth()/2)>arg0.getX()){
					heading=(((Math.PI)-Math.asin((arg0.getY()-(getHeight()/2))/(speed*zoom)))
							*180/Math.PI+90);
				}else{
					heading=(Math.asin((arg0.getY()-(getHeight()/2))/(speed*zoom))
							*180/Math.PI+90);
				}
				getControler().notifyHeadingSpeedChanged((int)heading, (int)speed);
			}
		}
		public void mouseMoved(MouseEvent arg0){}
	}
}
